Package: CalibrateSSB
Type: Package
Title: Weighting and Estimation for Panel Data with Non-Response
Version: 1.4.0
Date: 2026-05-21
Authors@R: c(person(given = "Øyvind", family = "Langsrud", role = c("aut", "cre"),
                    email = "oyl@ssb.no",
                    comment = c(ORCID = "0000-0002-1380-4396")))
Depends: R (>= 3.0.0), survey
Imports: methods
Suggests: ReGenesees, testthat (>= 2.1.0)
Description: Functions to calculate weights, estimates of changes and corresponding variance estimates for panel data with non-response. Partially overlapping samples are handled. Initially, weights are calculated by linear calibration. By default, the 'survey' package is used for this purpose. It is also possible to use 'ReGenesees', which can be installed from <https://github.com/DiegoZardetto/ReGenesees>. Variances of linear combinations (changes and averages) and ratios are calculated from a covariance matrix based on residuals according to the calibration model. The methodology was presented at the conference, The Use of R in Official Statistics, and is described in Langsrud (2016) <http://www.revistadestatistica.ro/wp-content/uploads/2016/06/RRS2_2016_A021.pdf>.  
License: GPL-2
Encoding: UTF-8
URL: https://github.com/statisticsnorway/ssb-calibratessb,
        https://statisticsnorway.github.io/ssb-calibratessb/
BugReports: https://github.com/statisticsnorway/ssb-calibratessb/issues
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-21 09:12:26 UTC; oyl
Author: Øyvind Langsrud [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1380-4396>)
Maintainer: Øyvind Langsrud <oyl@ssb.no>
Repository: CRAN
Date/Publication: 2026-05-21 10:22:03 UTC
Built: R 4.6.0; ; 2026-05-22 00:16:33 UTC; unix
