# Generated by roxygen2: do not edit by hand

S3method(print,calSSB)
S3method(print,calSSBwide)
export(AkuData)
export(CalSSBobj)
export(CalSSBobjReGenesees)
export(CalibrateSSB)
export(CalibrateSSBpanel)
export(CrossStrata)
export(LagDiff)
export(LinCombMatrix)
export(MatchVarNames)
export(OrderedVarNames)
export(PanelEstimation)
export(Period)
export(PeriodDiff)
export(WideFromCalibrate)
importFrom(methods,formalArgs)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,binomial)
importFrom(stats,glm)
importFrom(stats,lm)
importFrom(stats,lm.influence)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,predict)
importFrom(stats,resid)
importFrom(stats,update)
importFrom(stats,weights)
importFrom(survey,calibrate)
importFrom(survey,svydesign)
importFrom(utils,capture.output)
importFrom(utils,head)
importFrom(utils,tail)
