% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AkuData.R
\docType{data}
\name{testDataBasis}
\alias{testDataBasis}
\title{testDataBasis}
\description{
Data used by \code{\link{AkuData}}
}
\keyword{datasets}
\keyword{internal}
